/**
 * Program skeleton for the course "Programming embedded systems"
 */

#include <stdio.h>

/* Scheduler includes. */
#include "FreeRTOS.h"
#include "stm32f10x_conf.h"
#include "task.h"
#include "semphr.h"
#include "setup.h"
#include "assert.h"

#include "stm32f10x_lib.h"
#include "stm32f10x_exti.h"
#include "stm32f10x_gpio.h"
#include "stm32f10x_map.h"

/*-----------------------------------------------------------*/

xSemaphoreHandle interruptSignal;

/**
 * Immediate interrupt handler
 */
void vEXTI0Handler(void) {
  portBASE_TYPE higherPrio;

  // This will fail if the semaphore has already been given.
  // I.e., in this case an event might be dropped
  //
  // NB: it is not allowed to use the normal function
  //     xSemaphoreGive within an ISR!
  xSemaphoreGiveFromISR(interruptSignal, &higherPrio);

  // Clear pending-bit of interrupt
  EXTI_ClearITPendingBit( EXTI_Line0 );

  // FreeRTOS macro to signal the end of ISR
  portEND_SWITCHING_ISR(higherPrio);
}

/**
 * Scheduled interrupt handler
 */
void scheduledInterruptTask (void* params) {
  for (;;) {
    xSemaphoreTake(interruptSignal, portMAX_DELAY);
	printf("External interrupt (PA0, PB0, etc.)!\n");
  }
}

/*
 * Entry point of program execution
 */
int main( void )
{
  EXTI_InitTypeDef EXTI_InitStructure;
  GPIO_InitTypeDef GPIO_InitStructure;
  NVIC_InitTypeDef NVIC_InitStructure;

  prvSetupHardware();

  vSemaphoreCreateBinary(interruptSignal);
  // empty the semaphore (initially, no interrupt has occurred)
  xSemaphoreTake(interruptSignal, portMAX_DELAY);

  xTaskCreate(scheduledInterruptTask, "scheduled interrupt task",
              100, NULL, 1, NULL);

  /* Initialise data. */
  EXTI_DeInit();
  EXTI_StructInit( &EXTI_InitStructure );

  /* Configuration of external interrupt line 0 */
  EXTI_InitStructure.EXTI_Line = EXTI_Line0;
  EXTI_InitStructure.EXTI_Mode = EXTI_Mode_Interrupt;
  EXTI_InitStructure.EXTI_Trigger = EXTI_Trigger_Rising;
  EXTI_InitStructure.EXTI_LineCmd = ENABLE;
  EXTI_Init(&EXTI_InitStructure);

  /* Configuration of port pin PORTA0 (to which the
     interrupt line is connected */
  GPIO_InitStructure.GPIO_Pin = GPIO_Pin_0;
  GPIO_InitStructure.GPIO_Mode = GPIO_Mode_IPU;
  GPIO_Init( GPIOA, &GPIO_InitStructure );

  /* Configuration of the interrupt */
  NVIC_InitStructure.NVIC_IRQChannel = EXTI0_IRQChannel;
  NVIC_InitStructure.NVIC_IRQChannelSubPriority = 0;
  NVIC_InitStructure.NVIC_IRQChannelPreemptionPriority = 0;
  NVIC_InitStructure.NVIC_IRQChannelCmd = ENABLE;
  NVIC_Init( &NVIC_InitStructure );       

  printf("Setup complete\n");  // this is redirected to USART 1

  vTaskStartScheduler();
  assert(0);
  return 0;                 // not reachable
}
/*-----------------------------------------------------------*/
